//////////////////////////////////////////////////////////////////////////////////
//   hdmi color bar test                                                         //
//                                                                              //
//  Author: lhj                                                                 //
//                                                                              //
//          ALINX(shanghai) Technology Co.,Ltd                                  //
//          heijin                                                              //
//     WEB: http://www.alinx.cn/                                                //
//     BBS: http://www.heijin.org/                                              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
// Copyright (c) 2017,ALINX(shanghai) Technology Co.,Ltd                        //
//                    All rights reserved                                       //
//                                                                              //
// This source file may be used and distributed without restriction provided    //
// that this copyright statement is not removed from the file and that any      //
// derivative work contains the original copyright notice and the associated    //
// disclaimer.                                                                  //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

//================================================================================
//  Revision History:
//  Date          By            Revision    Change Description
//--------------------------------------------------------------------------------
//2019/08/30                    1.0          Original
//*******************************************************************************/
`timescale 1ns/1ps
module video_out_test
(
input                       sys_clk_p,          //system clock positive
input                       sys_clk_n,          //system clock negative 
input                       rst_n,              //reset ,low active
inout                       hdmi_scl,           //HDMI I2C clock
inout                       hdmi_sda,           //HDMI I2C data
output                      hdmi_nreset,        //9134 reset
output                      vout_hs,            //horizontal synchronization for 9134
output                      vout_vs,            //vertical synchronization for 9134
output                      vout_de,            //data valid for 9134
output                      vout_clk,           //clock for 9134
output[23:0]                vout_data           //data for 9134
);
wire                       video_clk;           //pixel clock
wire                       video_hs;            //horizontal synchronization
wire                       video_vs;            //vertical synchronization
wire                       video_de;            //video valid
wire[7:0]                  video_r;             //video red data
wire[7:0]                  video_g;             //video green data
wire[7:0]                  video_b;             //video blue data
wire                       sys_clk;             //single end clock   
wire[9:0]                  lut_index;           //look table index
wire[31:0]                 lut_data;            //look table data
assign vout_hs              = video_hs;
assign vout_vs              = video_vs;
assign vout_de              = video_de;
assign vout_clk             = video_clk;
assign vout_data            = {video_r,video_g,video_b};

wire                        locked;
wire                        done_9134;
assign hdmi_nreset           = locked;
/*************************************************************************
generate single end clock
**************************************************************************/
IBUFDS sys_clk_ibufgds
(
.O                          (sys_clk                  ),
.I                          (sys_clk_p                ),
.IB                         (sys_clk_n                )
);
/*************************************************************************
Generate the pixel clock required for the video
****************************************************************************/
sys_pll video_pll_m0
(
.clk_in1                    (sys_clk                  ),
.clk_out1                   (                         ),
.clk_out2                   (video_clk                ),
.reset                      (~rst_n                   ),
.locked                     (locked                   )
);
/*************************************************************************
Configure the register of 9011 and 9134
****************************************************************************/
i2c_config i2c_config_m0(
.rst                        (~locked                  ),
.clk                        (sys_clk                  ),
.clk_div_cnt                (16'd500                  ),
.i2c_addr_2byte             (1'b0                     ),
.lut_index                  (lut_index                ),
.lut_dev_addr               (lut_data[31:24]          ),
.lut_reg_addr               (lut_data[23:8]           ),
.lut_reg_data               (lut_data[7:0]            ),
.error                      (                         ),
.done                       (done_9134                ),
.i2c_scl                    (hdmi_scl                 ),
.i2c_sda                    (hdmi_sda                 )
);
/*************************************************************************
configure look-up table
****************************************************************************/
lut_hdmi lut_hdmi_m0(
.lut_index                  (lut_index                ),
.lut_data                   (lut_data                 )
);
/*************************************************************************
Call color bar generation module
****************************************************************************/
color_bar color_bar_m0(
.clk                        (video_clk                ),
.rst                        (~done_9134               ),
.hs                         (video_hs                 ),
.vs                         (video_vs                 ),
.de                         (video_de                 ),
.rgb_r                      (video_r                  ),
.rgb_g                      (video_g                  ),
.rgb_b                      (video_b                  )
);
endmodule 